@@ -19,11 +19,71 @@ sub get_parser {
         <autoaction: { [@item] }>
         startrule: config_line(s) { $item[1] }
         config_line:
-            interface
-            { $item[1] }
-            |
-            not_config
-            { $item[1] }
+            aaa { $item[1] } |
+            arp { $item[1] } |
+            bonding { $item[1] } |
+            clienv { $item[1] } |
+            core_dump { $item[1] } |
+            dns { $item[1] } |
+            domainname { $item[1] } |
+            format { $item[1] } |
+            hostname { $item[1] } |
+            inactivity_timeout { $item[1] } |
+            interface { $item[1] } |
+            ipv6 { $item[1] } |
+            ipv6_state { $item[1] } |
+            management { $item[1] } |
+            management { $item[1] } |
+            max_path_splits { $item[1] } |
+            message { $item[1] } |
+            net_access { $item[1] } |
+            ntp { $item[1] } |
+            ospf { $item[1] } |
+            password_controls { $item[1] } |
+            pbr { $item[1] } |
+            rip { $item[1] } |
+            snmp { $item[1] } |
+            static_route { $item[1] } |
+            timezone { $item[1] } |
+            tracefile { $item[1] } |
+            user { $item[1] } |
+            vrrp { $item[1] } |
+            web { $item[1] } |
+            not_config { $item[1] }
+
+        aaa: 'set aaa' { { type => $item[0], config => "yes" } }
+        arp: 'set arp' { { type => $item[0], config => "yes" } }
+        bonding: 'set bonding' { { type => $item[0], config => "yes" } }
+
+        clienv: 'set clienv' config_lock { type => $item[0], config => { @{ $item[2] } } }
+        config_lock: 'config-lock' m{on|off} { [$item[0], $item[2]] }
+
+
+        core_dump: 'set core-dump' { { type => $item[0], config => "yes" } }
+        dns: 'set dns' { { type => $item[0], config => "yes" } }
+        domainname: 'set domainname' { { type => $item[0], config => "yes" } }
+        format: 'set format' { { type => $item[0], config => "yes" } }
+        hostname: 'set hostname' { { type => $item[0], config => "yes" } }
+        inactivity_timeout: 'set inactivity-timeout' { { type => $item[0], config => "yes" } }
+        ipv6: 'set ipv6' { { type => $item[0], config => "yes" } }
+        ipv6_state: 'set ipv6-state' { { type => $item[0], config => "yes" } }
+        management: 'set management' { { type => $item[0], config => "yes" } }
+        max_path_splits: 'set max-path-splits'{ { type => $item[0], config => "yes" } }
+        message: 'set message' { { type => $item[0], config => "yes" } }
+        net_access: 'set net-access' { { type => $item[0], config => "yes" } }
+        ntp: 'set ntp' { { type => $item[0], config => "yes" } }
+        ospf: 'set ospf' { { type => $item[0], config => "yes" } }
+        password_controls: 'set password-controls' { { type => $item[0], config => "yes" } }
+        pbr: 'set pbr' { { type => $item[0], config => "yes" } }
+        rip: 'set rip' { { type => $item[0], config => "yes" } }
+        snmp: 'set snmp' { { type => $item[0], config => "yes" } }
+        static_route: 'set static-route' { { type => $item[0], config => "yes" } }
+        timezone: 'set timezone' { { type => $item[0], config => "yes" } }
+        tracefile: 'set tracefile' { { type => $item[0], config => "yes" } }
+        user: 'set user' { { type => $item[0], config => "yes" } }
+        vrrp: 'set vrrp' { { type => $item[0], config => "yes" } }
+        web: 'set web' { { type => $item[0], config => "yes" } }
+            
 
         interface: 
             'set interface' interface_name (ipv4_address_mask | vlan | state | comment | mtu | auto_negotiation | link_speed)
@@ -37,7 +97,7 @@ sub get_parser {
 
         vlan: 'vlan' m{\d+} { $return = { $item[0], $item[2] } }
         state: 'state' m{\S+} { $return = { $item[0], $item[2] } }
-        comment: 'comments' m{["\S]+} { $return = { $item[0], $item[2] } }
+        comment: 'comments' m{"[\w\s]+"} { $return = { $item[0], $item[2] } }
         mtu: 'mtu' m{\d+} { $return = { $item[0], $item[2] } }
         auto_negotiation: 'auto-negotiation' m{\S+} { $return = { $item[0], $item[2] } }
         link_speed: 'link-speed' m{\S+} { $return = { $item[0], $item[2] } }
@@ -73,23 +133,37 @@ sub get_output_drivers {
 sub post_process {
     my ($parsed_config) = @_;
     my %aggregation = ();
+    my %post_processed_config;
 
+    # For each 'type' of config, (e.g. interface config), the aggregator key we're using to aggregate the separate
+    # config lines together into a single hash.
     my $aggregator_keys_for = {
         interface => q{$config_entry->{config}->{name}},
         not_config => q{undef},
     };
-        my @pushover;
 
+    # Go through each config entry (which was originally each line of config. If there's an aggregate key defined, 
+    # aggregate on the 'type' and then this 'key'. 
+    #
+    # If not, then just push it to the post processed hash.
     for my $config_entry (@{ $parsed_config }) {
         my $aggregate_key = eval $aggregator_keys_for->{ $config_entry->{type} };
         if (defined $aggregate_key) {
             @{ $aggregation{ $config_entry->{type} }{ $aggregate_key } }{ keys %{ $config_entry->{config} } } = values %{ $config_entry->{config} };
         } else {
-            push @{ $aggregation{ $config_entry->{type} } }, $config_entry->{config};
+            push @{ $post_processed_config{ $config_entry->{type} } }, $config_entry->{config};
         }
     }
 
-    $parsed_config = \%aggregation;
+    # It's of the form $aggregation{type}{key} = { #interface into }; but the key is implicitly part of the hash it points to.
+    # Turn the hash of hash of hashes into a hash of array of hashes ( $aggregation{type} = [ { #interface info } ];
+    for my $config_type (keys %aggregation) {
+        $aggregation{ $config_type } = [ values %{ $aggregation{ $config_type } } ];
+    }
+
+    @post_processed_config{ keys %aggregation } = values %aggregation;
+
+    return \%post_processed_config;
 }
 
 
@@ -97,41 +171,52 @@ sub post_process {
 sub csv_output_driver {
     my ($fh, $filename, $parsed_config) = @_;
     my $csv_type_driver = { 
-        interface => \&_csv_interface_driver,
+        interface       => \&_csv_interface_driver,
+        not_config      => \&_csv_not_config_driver,
     };
 
     say "=" x 16 . "BEGIN FILE $filename" . "=" x 16;
 
     TYPE:
     for my $type (keys %{ $parsed_config }) {
-        say "=" x 16 . "BEGIN TYPE $type" . "=" x 16;
+        say "-" x 8 . "BEGIN TYPE $type" . "-" x 8;
 
         defined $csv_type_driver->{$type} ? 
             $csv_type_driver->{$type}->($fh, $parsed_config->{$type}) :
             warn "No CSV output driver for $type\n" and next TYPE;
 
-        say "=" x 16 . "END TYPE $type" . "=" x 16;
+        say "-" x 8 . "END TYPE $type" . "-" x 8;
     }
 
-    say "=" x 16 . "END FILE $filename" . "=" x 16;
+    say "-" x 8 . "END FILE $filename" . "-" x 8;
 }
 
 sub _csv_interface_driver {
-    my $fh = $_[0];
-    my %interfaces = %{ $_[1] };
-    my @interface_properties = qw{name state vlan ipv4_address ipv4_mask auto_negotiation link_speed mtu comment};
+    my ($fh, $interfaces_ref) = @_;
 
-    
+    # Print the CSV schema line
+    my @interface_properties = qw{name state vlan ipv4_address ipv4_mask auto_negotiation link_speed mtu comment};
     say $fh join(',', @interface_properties);
-    for my $interface (keys %interfaces) {
-        my @properties = @{ $interfaces{$interface} }{@interface_properties};
+
+    # Interface through the interfaces, extract and print their properties
+    for my $interface (@{ $interfaces_ref }) {
+        my @properties = @{ $interface }{ @interface_properties };
 
         # Replace any undef with an empty string
-        @properties = map { defined $_ ? $_ : '' } @properties;
+        @properties =  map { defined $_ ? $_ : '' } @properties;
         say $fh join(',', @properties);
     }
 }
 
+sub _csv_not_config_driver {
+    my ($fh, $not_config) = @_;
+
+    for my $config_line (@{ $not_config }) {
+        print $fh "$config_line\n";
+    }
+}
+
+
 
 
 
@@ -157,7 +242,7 @@ Device::Network::ConfigParser::CheckPoint::Gaia - Parse CheckPoint Configuration
 
 =head1 VERSION
 
-version 0.001
+version 0.002
 
 =head1 SYNOPSIS
 
